Class: Watson::Remote::Bitbucket

Inherits:
Object
  • Object
show all
Extended by:
Watson
Defined in:
lib/watson/bitbucket.rb

Overview

Bitbucket remote access class Contains all necessary methods to obtain access to, get issue list, and post issues to Bitbucket

Constant Summary collapse

DEBUG =

Debug printing for this class

false

Constants included from Watson

BLUE, BOLD, CYAN, GLOBAL_DEBUG_OFF, GLOBAL_DEBUG_ON, GRAY, GREEN, MAGENTA, Watson::RED, Watson::RESET, UNDERLINE, VERSION, WHITE, YELLOW

Class Method Summary collapse

Methods included from Watson

check_less, debug_print

Class Method Details

.get_issues(config) ⇒ Object

Get all remote Bitbucket issues and store into Config container class



169
170
171
172
173
174
175
176
177
178
179
180
181
182
183
184
185
186
187
188
189
190
191
192
193
194
195
196
197
198
199
200
201
202
203
204
205
206
207
208
209
210
211
212
213
214
215
216
217
218
219
220
221
222
223
224
225
226
227
228
229
230
231
232
233
234
235
236
237
238
239
240
241
242
243
244
245
246
247
248
249
250
251
252
253
254
255
256
257
258
259
260
261
262
263
264
265
266
267
# File 'lib/watson/bitbucket.rb', line 169

def get_issues(config)

  # Identify method entry
  debug_print "#{ self.class } : #{ __method__ }\n"


  # Set up formatter for printing errors
  # config.output_format should be set based on less status by now
  formatter = Printer.new(config).build_formatter


  # Only attempt to get issues if API is specified
  if config.bitbucket_api.empty?
    debug_print "No API found, this shouldn't be called...\n"
    return false
  end

  # If we haven't obtained the pw from user yet, do it
  if config.bitbucket_pw.empty?
    # No OAuth for Bitbucket yet, gotta get user password in order to make calls :(
    formatter.print_status "!", YELLOW
    print BOLD + "Bitbucket password required for remote checking/posting.\n" + RESET
    print "      Password: "

    # Block output to tty to prevent PW showing, Linux/Unix only :(
    system "stty -echo"
    _password = $stdin.gets.chomp
    system "stty echo"
    if _password.empty?
      print "Input is blank. Please enter your password!\n"
      return false
    else
      print "\n"
    end

    config.bitbucket_pw = _password
  end


  # Get all open tickets (anything but resolved)
  # Create options hash to pass to Remote::http_call
  # Issues URL for Bitbucket + SSL
  opts = {:url        => "https://bitbucket.org/api/1.0/repositories/#{ config.bitbucket_repo }/issues?status=!resolved",
      :ssl        => true,
      :method     => "GET",
      :basic_auth => [config.bitbucket_api, config.bitbucket_pw],
      :verbose    => false
       }

  _json, _resp  = Watson::Remote.http_call(opts)


  # Check response to validate repo access
  if _resp.code != "200"
    formatter.print_status "x", RED
    print BOLD + "Unable to access remote #{ config.bitbucket_repo }, Bitbucket API may be invalid\n" + RESET
    print "      Make sure you have created an issue tracker for your repository on the Bitbucket website\n"
    print "      Consider running --remote (-r) option to regenerate/validate settings\n"
    print "      Status: #{ _resp.code } - #{ _resp.message }\n\n"

    debug_print "Bitbucket invalid, setting config var\n"
    config.bitbucket_valid = false
    return false
  end



  config.bitbucket_issues[:open] = _json["issues"].empty? ? Hash.new : _json["issues"]
  config.bitbucket_valid = true

  # Get all closed tickets
  # Create options hash to pass to Remote::http_call
  # Issues URL for Bitbucket + SSL
  opts = {:url        => "https://bitbucket.org/api/1.0/repositories/#{ config.bitbucket_repo }/issues?status=resolved",
      :ssl        => true,
      :method     => "GET",
      :basic_auth => [config.bitbucket_api, config.bitbucket_pw],
      :verbose    => false
       }

  _json, _resp  = Watson::Remote.http_call(opts)

  # Check response to validate repo access
  # Shouldn't be necessary if we passed the last check but just to be safe
  if _resp.code != "200"
    formatter.print_status "x", RED
    print BOLD + "Unable to get closed issues.\n" + RESET
    print "      Since the open issues were obtained, something is probably wrong and you should file a bug report or something...\n"
    print "      Status: #{ _resp.code } - #{ _resp.message }\n"

    debug_print "Bitbucket invalid, setting config var\n"
    config.bitbucket_valid = false
    return false
  end

  config.bitbucket_issues[:closed] = _json["issues"].empty? ? Hash.new : _json["issues"]
  config.bitbucket_valid = true
  return true
end

.post_issue(issue, config) ⇒ Object

Post given issue to remote Bitbucket repo



272
273
274
275
276
277
278
279
280
281
282
283
284
285
286
287
288
289
290
291
292
293
294
295
296
297
298
299
300
301
302
303
304
305
306
307
308
309
310
311
312
313
314
315
316
317
318
319
320
321
322
323
324
325
326
327
328
329
330
331
332
333
334
335
336
337
338
339
340
341
342
343
344
345
346
347
348
349
350
351
352
353
354
355
356
357
358
359
360
361
362
363
364
365
366
367
368
369
370
371
372
373
374
375
376
377
# File 'lib/watson/bitbucket.rb', line 272

def post_issue(issue, config)
# [todo] - Better way to identify/compare remote->local issues than md5
#        Current md5 based on some things that easily can change, need better ident

  # Identify method entry
  debug_print "#{self.class} : #{__method__}\n"


  # Set up formatter for printing errors
  # config.output_format should be set based on less status by now
  formatter = Printer.new(config).build_formatter


  # Only attempt to get issues if API is specified
  if config.bitbucket_api.empty?
    debug_print "No API found, this shouldn't be called...\n"
    return false
  end

  # Check that issue hasn't been posted already by comparing md5s
  # Go through all open issues, if there is a match in md5, return out of method
  # [todo] - Play with idea of making body of GitHub issue hash format to be exec'd
  #      Store pieces in text as :md5 => "whatever" so when we get issues we can
  #      call exec and turn it into a real hash for parsing in watson
  #      Makes watson code cleaner but not as readable comment on GitHub...?
  debug_print "Checking open issues to see if already posted\n"
  config.bitbucket_issues[:open].each do | _open |
    if _open["content"].include?(issue[:md5])
      debug_print "Found in #{ _open["title"] }, not posting\n"
      return false
    end
    debug_print "Did not find in #{_open["title"]}\n"
  end

  debug_print "Checking closed issues to see if already posted\n"
  config.bitbucket_issues[:closed].each do  | _closed |
    if _closed["content"].include?(issue[:md5])
      debug_print "Found in #{ _closed["title"] }, not posting\n"
      return false
    end
    debug_print "Did not find in #{ _closed["title"] }\n"
  end


  # If we haven't obtained the pw from user yet, do it
  if config.bitbucket_pw.empty?
    # No OAuth for Bitbucket yet, gotta get user password in order to make calls :(
    formatter.print_status "!", YELLOW
    print BOLD + "Bitbucket password required for remote checking/posting.\n" + RESET
    print "      Password: "

    # Block output to tty to prevent PW showing, Linux/Unix only :(
    print "Password: "
    system "stty -echo"
    _password = $stdin.gets.chomp
    system "stty echo"
    if _password.empty?
      print "Input is blank. Please enter your password!\n"
      return false
    else
      print "\n"
    end

    config.bitbucket_pw = _password
  end




  # We didn't find the md5 for this issue in the open or closed issues, so safe to post

  # Create the body text for the issue here, too long to fit nicely into opts hash
  # [review] - Only give relative path for privacy when posted
  _body = "__filename__ : #{ issue[:path] }  \n" +
      "__line #__ : #{ issue[:line_number] }  \n" +
      "__tag__ : #{ issue[:tag] }  \n" +
      "__md5__ : #{ issue[:md5] }  \n\n" +
      "#{ issue[:context].join }"

  # Create option hash to pass to Remote::http_call
  # Issues URL for GitHub + SSL
  # No tag or label concept in Bitbucket unfortunately :(
  opts = {:url        => "https://bitbucket.org/api/1.0/repositories/#{ config.bitbucket_repo }/issues",
      :ssl        => true,
      :method     => "POST",
      :basic_auth => [config.bitbucket_api, config.bitbucket_pw],
      :data   => [{"title" => issue[:title] + " [#{ issue[:path] }]",
              "content" => _body }],
      :verbose    => false
       }

  _json, _resp  = Watson::Remote.http_call(opts)


  # Check response to validate repo access
  # Shouldn't be necessary if we passed the last check but just to be safe
  if _resp.code != "200"
    formatter.print_status "x", RED
    print BOLD + "Post unsuccessful. \n" + RESET
    print "      Since the open issues were obtained earlier, something is probably wrong and you should let someone know...\n"
    print "      Status: #{ _resp.code } - #{ _resp.message }\n"
    return false
  end

  return true
end

.setup(config) ⇒ Object

Setup remote access to Bitbucket Get Username, Repo, and PW and perform necessary HTTP calls to check validity



22
23
24
25
26
27
28
29
30
31
32
33
34
35
36
37
38
39
40
41
42
43
44
45
46
47
48
49
50
51
52
53
54
55
56
57
58
59
60
61
62
63
64
65
66
67
68
69
70
71
72
73
74
75
76
77
78
79
80
81
82
83
84
85
86
87
88
89
90
91
92
93
94
95
96
97
98
99
100
101
102
103
104
105
106
107
108
109
110
111
112
113
114
115
116
117
118
119
120
121
122
123
124
125
126
127
128
129
130
131
132
133
134
135
136
137
138
139
140
141
142
143
144
145
146
147
148
149
150
151
152
153
154
155
156
157
158
159
160
161
162
163
164
# File 'lib/watson/bitbucket.rb', line 22

def setup(config)

  # Identify method entry
  debug_print "#{ self.class } : #{ __method__ }\n"

  formatter = Printer.new(config).build_formatter
  formatter.print_status "+", GREEN
  print BOLD +  "Attempting to access Bitbucket...\n" + RESET

  # Check config to make sure no previous repo info exists
  unless config.bitbucket_api.empty? && config.bitbucket_repo.empty?
    formatter.print_status "!", RED
    print BOLD + "Previous Bitbucket API + Repo is in RC, are you sure you want to overwrite?\n" + RESET
    print "      (Y)es/(N)o: "

    # Get user input
    _overwrite = $stdin.gets.chomp
    if ["no", "n"].include?(_overwrite.downcase)
      print "\n"
      formatter.print_status "x", RED
      print BOLD + "Not overwriting current Bitbucket API + repo info\n" + RESET
      return false
    end
  end


  formatter.print_status "!", YELLOW
  print BOLD + "Access to your Bitbucket account required to make/update issues\n" + RESET
  print "      See help or README for more details on GitHub/Bitbucket access\n\n"


  # [todo] - Bitbucket OAuth not implemented yet so warn user about HTTP Auth
  # Bitbucket doesn't have nonOAuth flow that GitHub does :(
  # Even if I use OAuth lib, still need to validate from webview which is lame
  formatter.print_status "!", RED
  print BOLD + "Bitbucket OAuth not implemented yet.\n" + RESET;
  print "      Basic HTTP Auth in use, will request PW entry every time.\n\n"


  # [todo] - Don't just check for blank password but invalid as well
  # Poor mans username/password grabbing
  print BOLD + "Username: " + RESET
  _username = $stdin.gets.chomp
  if _username.empty?
    formatter.print_status "x", RED
    print BOLD + "Input blank. Please enter your username!\n\n" + RESET
    return false
  end

  print "\n"

  # Get repo information, if blank give error
  formatter.print_status "!", YELLOW
  print BOLD + "Repo information required\n" + RESET
  print "      Please provide owner that repo is under followed by repo name\n"
  print "      e.g. owner: nhmood, repo: watson (case sensitive)\n"
  print "      See help or README for more details on GitHub access\n\n"

  print BOLD + "Owner: " + RESET
  _owner = $stdin.gets.chomp
  if _owner.empty?
    print "\n"
    formatter.print_status "x", RED
    print BOLD + "Input blank. Please enter the owner the repo is under!\n\n" + RESET
    return false
  end

  print BOLD + "Repo: " + RESET
  _repo = $stdin.gets.chomp
  if _repo.empty?
    print "\n"
    formatter.print_status "x", RED
    print BOLD + "Input blank. Please enter the repo name!\n\n" + RESET
    return false
  end

  print "\n"

  # [fix] - Crossplatform password block needed, not sure if current method is safe either
  # Block output to tty to prevent PW showing, Linux/Unix only :(
  print BOLD + "Password: " + RESET
  system "stty -echo"
  _password = $stdin.gets.chomp
  system "stty echo"
  print "\n"
  if _password.empty?
    formatter.print_status "x", RED
    print BOLD + "Input is blank. Please enter your password!\n\n" + RESET
    return false
  end

  # HTTP Request to check if Repo exists and user has access
  # http://confluence.atlassian.com/display/BITBUCKET/Use+the+Bitbucket+REST+APIs

  # Create options hash to pass to Remote::http_call
  # Endpoint for accessing Repo as User with SSL
  # Basic auth with user input
  opts = {:url        => "https://bitbucket.org/api/1.0/repositories/#{_owner}/#{_repo}",
      :ssl        => true,
      :method     => "GET",
      :basic_auth => [_username, _password],
      :verbose    => false
       }

  _json, _resp  = Watson::Remote.http_call(opts)

  # Check response to validate authorization
  if _resp.code == "200"
    print "\n"
    formatter.print_status "o", GREEN
    print BOLD + "Successfully accessed remote repo with given credentials\n" + RESET
  else
    print "\n"
    formatter.print_status "x", RED
    print BOLD + "Unable to access /#{ _owner }/#{ _repo } with given credentials\n" + RESET
    print "      Check that credentials are correct and repository exists under user\n"
    print "      Status: #{ _resp.code } - #{ _resp.message }\n\n"
    return false
  end


  # No OAuth for Bitbucket yet so just store username in api for config
  # This will let us just prompt for PW
  config.bitbucket_api = _username
  config.bitbucket_pw = _password # Never gets written to file
  config.bitbucket_repo = "#{ _owner }/#{ _repo }"
  debug_print " \n"

  # All setup has been completed, need to update RC
  # Call config updater/writer from @config to write config
  debug_print "Updating config with new Bitbucket info\n"
  config.update_conf("bitbucket_api", "bitbucket_repo")

  print "\n"
  formatter.print_status "o", GREEN
  print BOLD + "Bitbucket successfully setup\n" + RESET
  print "      Issues will now automatically be retrieved from Bitbucket by default\n"
  print "      Use -u, --update to post issues to GitHub\n"
  print "      See help or README for more details on GitHub/Bitbucket access\n\n"

  return true

end